/**
 * Copyright (C) 2007-2012 Linutop SAS <http://www.linutop.com/>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef LIBLUSM_H
#define LIBLUSM_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <glib.h>
#include <glib/gi18n-lib.h>
#include <gio/gio.h>

#include <dbus/dbus-glib-lowlevel.h>
#include <dbus/dbus-glib.h>

typedef gint (*LusmProgressFunc) (void *ptr, double dltotal, double dlnow,
                                             double ultotal, double ulnow);

enum {
    LUSM_STATUS_PREP = 0,
    LUSM_STATUS_COPY,
    LUSM_STATUS_SIMPLE_COPY,
    LUSM_STATUS_DOWNLOAD_LIVEFS, /* lusm-web */
    LUSM_STATUS_DOWNLOAD_SQUASHFS, /* lusm-web */
    LUSM_STATUS_VERIF,
    LUSM_STATUS_USER,
    LUSM_STATUS_EXTRA_USER, /* lusm-web */
    LUSM_STATUS_CONFIG,
    LUSM_STATUS_BOOT,
    LUSM_STATUS_SYNC,
    LUSM_STATUS_SUCCESS,
    LUSM_STATUS_FAILURE
};

struct lusm_status_struct {
    const gchar *msg;
    gdouble begin;
    gdouble end;
};

extern struct lusm_status_struct lusm_status_desc[];

typedef struct _Device {
    DBusGProxy *proxy;
    const gchar *path;
    const gchar *vendor;
    const gchar *model;
    guint64 size;
} Device;

typedef struct _Version {
    const gchar *id;
    const gchar *text;
} Version;

gboolean lusm_init     ();
void     lusm_shutdown ();

Device *lusm_device_from_uuid    (const gchar *uuid);
Device *lusm_device_from_proxy   (DBusGProxy *device_proxy);
Device *lusm_get_device_for_path (const gchar *device_str);

void lusm_device_free (Device *device);

gboolean lusm_partition (Device *device,
                         gboolean vfat);

gboolean lusm_simple_duplicate (GFileProgressCallback callback);
gboolean lusm_duplicate        (GFileProgressCallback callback);
gboolean lusm_copy_data        (GFileProgressCallback callback);

gboolean lusm_download_livefs   (const gchar *source,
                                 LusmProgressFunc progress_func,
                                 gpointer data);
gboolean lusm_download_squashfs (const gchar *source,
                                 LusmProgressFunc progress_func,
                                 gpointer data);
gboolean lusm_download_extra    (const gchar *source,
                                 LusmProgressFunc progress_func,
                                 gpointer data);

gboolean lusm_download_check_md5  (const gchar *source);
gboolean lusm_check_md5_from_file (const gchar *files_list);

gboolean lusm_setup        (gboolean backup);
gboolean lusm_set_bootable (Device *device);

gboolean lusm_rsync ();

GList *lusm_available_targets ();

void lusm_connect_to_add_remove      (GCallback on_add_handler,
                                      GCallback on_remove_handler);
void lusm_disconnect_from_add_remove ();

#endif /* LIBLUSM_H */
